@@ -614,9 +614,9 @@ compilation_group.add_option(
     metavar="YAML_FILENAME",
     help="""\
 User provided Yaml file with package configuration. You can include DLLs,
-remove bloat, add hidden dependencies. Check User Manual for a complete
-description of the format to use. Can be given multiple times. Defaults
-to empty.""",
+remove bloat, add hidden dependencies. Check the Nuitka Package Configuration
+Manual for a complete description of the format to use. Can be given
+multiple times. Defaults to empty.""",
 )
 
 compilation_group.add_option(
@@ -684,8 +684,7 @@ choice, also not for standalone mode and using it will be an error. This may
 include path information that needs to exist though. Defaults to '%s' on this
 platform.
 """
-    % "<program_name>"
-    + (".exe" if isWin32OrPosixWindows() else ".bin"),
+    % ("<program_name>" + (".exe" if isWin32OrPosixWindows() else ".bin")),
 )
 
 output_group.add_option(
@@ -1399,6 +1398,18 @@ del windows_group
 
 macos_group = parser.add_option_group("macOS specific controls")
 
+macos_group.add_option(
+    "--macos-create-app-bundle",
+    action="store_true",
+    dest="macos_create_bundle",
+    default=False,
+    help="""\
+When compiling for macOS, create a bundle rather than a plain binary
+application. This is the only way to unlock the disabling of console,
+get high DPI graphics, etc. and implies standalone mode. Defaults to
+off.""",
+)
+
 macos_group.add_option(
     "--macos-target-arch",
     action="store",
@@ -1412,17 +1423,6 @@ is what the running Python allows for. Default is "native" which is
 the architecture the Python is run with.""",
 )
 
-macos_group.add_option(
-    "--macos-create-app-bundle",
-    action="store_true",
-    dest="macos_create_bundle",
-    default=False,
-    help="""\
-When compiling for macOS, create a bundle rather than a plain binary
-application. Currently experimental and incomplete. Currently this
-is the only way to unlock disabling of console.Defaults to off.""",
-)
-
 macos_group.add_option(
     "--macos-app-icon",
     action="append",
@@ -2021,12 +2021,15 @@ def parseOptions(logger):
     filename_args = []
     module_mode = False
 
+    # Options may be coming from GitHub workflow configuration as well.
+    _considerGithubWorkflowOptions(phase="early")
+
     for count, arg in enumerate(sys.argv):
         if count == 0:
             continue
 
-        if arg.startswith("--main="):
-            filename_args.append(arg)
+        if arg.startswith(("--main=", "--script-name=")):
+            filename_args.append(arg.split("=", 1)[1])
 
         if arg == "--module":
             module_mode = True
@@ -2042,9 +2045,6 @@ def parseOptions(logger):
             + sys.argv[1:]
         )
 
-    # Options may be coming from GitHub workflow configuration as well.
-    _considerGithubWorkflowOptions(phase="early")
-
     # Next, lets activate plugins early, so they can inject more options to the parser.
     _considerPluginOptions(logger)
 
