@@ -28,7 +28,6 @@ module.
 
 import os
 
-from nuitka.Options import shallUseStaticLibPython
 from nuitka.PythonVersions import python_version
 from nuitka.utils.FileOperations import getFileContents, isFilenameBelowPath
 from nuitka.utils.ModuleNames import ModuleName
@@ -104,10 +103,13 @@ def getStandardLibraryPaths():
             if os.path.isdir(candidate):
                 stdlib_paths.add(candidate)
 
-        if isWin32OrPosixWindows() and not shallUseStaticLibPython():
-            import _ctypes
+        if isWin32OrPosixWindows():
+            from nuitka.Options import shallUseStaticLibPython
 
-            stdlib_paths.add(os.path.dirname(_ctypes.__file__))
+            if not shallUseStaticLibPython():
+                import _ctypes
+
+                stdlib_paths.add(os.path.dirname(_ctypes.__file__))
 
         getStandardLibraryPaths.result = [
             os.path.normcase(os.path.normpath(stdlib_path))
@@ -117,18 +119,18 @@ def getStandardLibraryPaths():
     return getStandardLibraryPaths.result
 
 
-def isStandardLibraryPath(filename):
-    """Check if a path is in the standard library."""
-
-    filename = os.path.normcase(os.path.normpath(filename))
-
+def _isStandardLibraryPath(filename):
     # In virtualenv, the "site.py" lives in a place that suggests it is not in
     # standard library, although it is.
     if os.path.basename(filename) == "site.py":
         return True
 
     # These never are in standard library paths.
-    if "dist-packages" in filename or "site-packages" in filename:
+    if (
+        "dist-packages" in filename
+        or "site-packages" in filename
+        or "vendor-packages" in filename
+    ):
         return False
 
     for candidate in getStandardLibraryPaths():
@@ -138,6 +140,20 @@ def isStandardLibraryPath(filename):
     return False
 
 
+_is_standard_library_path_cache = {}
+
+
+def isStandardLibraryPath(filename):
+    """Check if a path is in the standard library."""
+
+    filename = os.path.normcase(os.path.normpath(filename))
+
+    if filename not in _is_standard_library_path_cache:
+        _is_standard_library_path_cache[filename] = _isStandardLibraryPath(filename)
+
+    return _is_standard_library_path_cache[filename]
+
+
 # Some modules we want to exclude entirely.
 _excluded_stdlib_modules = ["__main__.py", "__init__.py", "antigravity.py"]
 
@@ -161,6 +177,8 @@ def scanStandardLibraryPath(stdlib_dir):
                 dirs.remove("site-packages")
             if "dist-packages" in dirs:
                 dirs.remove("dist-packages")
+            if "vendor-packages" in dirs:
+                dirs.remove("vendor-packages")
             if "test" in dirs:
                 dirs.remove("test")
             if "turtledemo" in dirs:
@@ -257,6 +275,7 @@ _stdlib_no_auto_inclusion_list = (
     "_curses_panel",
     "sqlite3",
     "_sqlite3",
+    "shelve",
     "dbm",
     "_dbm",
     "bdb",
@@ -275,12 +294,16 @@ _stdlib_no_auto_inclusion_list = (
     "_pydecimal",
     "_decimal",
     "statistics",
+    "csv",
+    "_csv",
     "lzma",
     "_lzma",
     "bz2",
     "_bz2",
     "logging",
+    "tempfile",
     "subprocess",
+    "_posixsubprocess",
     "socket",
     "selectors",
     "select",
@@ -303,8 +326,30 @@ _stdlib_no_auto_inclusion_list = (
     "runpy",
     "lib2to3",
     "doctest",
-    "email",
+    # Optional dependency of json need not by collect by itself, but usage will
+    # go through "json.encoder/json.decoder/json.scanner" of course.
+    "_json",
+    # Optional dependency of "bisect" need not by collect by itself, but usage will
+    # go through "bisect" of course.
+    "_bisect",
+    # Optional dependency of "heapq" need not by collect by itself, but usage will
+    # go through "heapq" of course.
+    "_heapq",
+    # Dependency of crypt, that may not be used, requiring this to be explicit.
+    "_crypt",
+    # Dependency of contextvars, that may not be used, requiring this to be explicit.
+    "_contextvars",
+    # Dependency of random, that may not be used, requiring this to be explicit.
+    "random",
+    # Avoid this one if not built-in, since it's an extension module.
+    "array",
+    # Runners for programs
+    "json.tool",
+    "zipapp",
     "tabnanny",
+    # Packages that will be imported rarely by extension modules
+    "email",
+    "mailbox",
     "argparse",
     "telnetlib",
     "smtplib",
@@ -316,7 +361,16 @@ _stdlib_no_auto_inclusion_list = (
     "select",
     "wsgiref",
     "sunau",
+    "aifc",
+    "wave",
+    "audioop",
+    "getpass",
+    "grp",
+    "pty",
+    "tty",
+    "termios",
     "this",
+    "textwrap",
     # Distribution and bytecode related stuff
     "plistlib",
     "distutils",
@@ -324,6 +378,7 @@ _stdlib_no_auto_inclusion_list = (
     "venv",
     "py_compile",
     "msilib",
+    "_opcode",
     # tzdata is not always needed
     "zoneinfo",
     # tkinter under all its names
