@@ -23,22 +23,22 @@ that to be done and causing massive degradations.
 """
 
 import ast
-import os
 import re
-import sys
 
 from nuitka.containers.OrderedDicts import OrderedDict
 from nuitka.Errors import NuitkaForbiddenImportEncounter
-from nuitka.plugins.PluginBase import NuitkaPluginBase
+from nuitka.ModuleRegistry import getModuleByName
+from nuitka.plugins.PluginBase import NuitkaYamlPluginBase
 from nuitka.utils.ModuleNames import ModuleName
-from nuitka.utils.Yaml import getYamlPackageConfiguration
 
-# spell-checker: ignore dask,numba,statsmodels,matplotlib
+# spell-checker: ignore dask,numba,statsmodels,matplotlib,sqlalchemy,ipykernel
 
 _mode_choices = ("error", "warning", "nofollow", "allow")
 
 
-class NuitkaPluginAntiBloat(NuitkaPluginBase):
+class NuitkaPluginAntiBloat(NuitkaYamlPluginBase):
+    # Lots of details, a bunch of state is cached, pylint: disable=too-many-instance-attributes
+
     plugin_name = "anti-bloat"
     plugin_desc = (
         "Patch stupid imports out of widely used library modules source codes."
@@ -62,6 +62,8 @@ class NuitkaPluginAntiBloat(NuitkaPluginBase):
     ):
         # Many details, due to many repetitive arguments, pylint: disable=too-many-branches,too-many-statements
 
+        NuitkaYamlPluginBase.__init__(self)
+
         self.show_changes = show_changes
 
         # Default manually to default argument value:
@@ -78,8 +80,6 @@ class NuitkaPluginAntiBloat(NuitkaPluginBase):
         if noinclude_numba_mode is None:
             noinclude_numba_mode = noinclude_default_mode
 
-        self.config = getYamlPackageConfiguration()
-
         self.handled_modules = OrderedDict()
 
         # These should be checked, to allow disabling anti-bloat contents.
@@ -91,6 +91,10 @@ class NuitkaPluginAntiBloat(NuitkaPluginBase):
                 noinclude_setuptools_mode,
                 "setuptools",
             )
+            self.handled_modules["triton"] = (
+                noinclude_setuptools_mode,
+                "setuptools",
+            )
         else:
             self.control_tags["use_setuptools"] = True
 
@@ -104,21 +108,47 @@ class NuitkaPluginAntiBloat(NuitkaPluginBase):
                 noinclude_pytest_mode,
                 "pytest",
             )
+            self.handled_modules["sqlalchemy.testing"] = (
+                noinclude_pytest_mode,
+                "pytest",
+            )
+            self.handled_modules["distributed.utils_test"] = (
+                noinclude_pytest_mode,
+                "pytest",
+            )
         else:
             self.control_tags["use_pytest"] = True
 
         if noinclude_unittest_mode != "allow":
             self.handled_modules["unittest"] = noinclude_unittest_mode, "unittest"
             self.handled_modules["doctest"] = noinclude_unittest_mode, "unittest"
+            self.handled_modules["test.support"] = noinclude_unittest_mode, "unittest"
+            self.handled_modules["test.test_support"] = (
+                noinclude_unittest_mode,
+                "unittest",
+            )
+            self.handled_modules["future.moves.test.support"] = (
+                noinclude_unittest_mode,
+                "unittest",
+            )
         else:
             self.control_tags["use_unittest"] = True
 
         if noinclude_ipython_mode != "allow":
             self.handled_modules["IPython"] = noinclude_ipython_mode, "IPython"
+            self.handled_modules["ipykernel"] = noinclude_ipython_mode, "IPython"
+            self.handled_modules["jupyter_client"] = (
+                noinclude_ipython_mode,
+                "IPython",
+            )
             self.handled_modules["matplotlib_inline.backend_inline"] = (
                 noinclude_ipython_mode,
                 "IPython",
             )
+            self.handled_modules["altair._magics"] = (
+                noinclude_ipython_mode,
+                "IPython",
+            )
         else:
             self.control_tags["use_ipython"] = True
 
@@ -176,6 +206,16 @@ form 'module_name:[%s]'."""
 
         self.warnings_given = set()
 
+        # Keep track of modules prevented from automatically following and the
+        # information given for that.
+        self.no_auto_follows = {}
+
+        # Keep track of modules prevented from being followed at all.
+        self.no_follows = OrderedDict()
+
+        # Cache execution context for anti-bloat configs.
+        self.context_codes = {}
+
     def getEvaluationConditionControlTags(self):
         return self.control_tags
 
@@ -290,24 +330,36 @@ which can and should be a top level package and then one choice, "error",
 "warning", "nofollow", e.g. PyQt5:error.""",
         )
 
+    def _getContextCode(self, module_name, anti_bloat_config):
+        context_code = anti_bloat_config.get("context", "")
+        if type(context_code) in (tuple, list):
+            context_code = "\n".join(context_code)
+
+        if context_code not in self.context_codes:
+            context = {}
+
+            try:
+                # We trust the yaml files, pylint: disable=exec-used
+                exec(context_code, context)
+            except Exception as e:  # pylint: disable=broad-except
+                self.sysexit(
+                    """\
+Error, cannot exec module '%s', context code '%s' due to: %s"""
+                    % (module_name, context_code, e)
+                )
+
+            self.context_codes[context_code] = context
+
+        return dict(self.context_codes[context_code])
+
     def _onModuleSourceCode(self, module_name, anti_bloat_config, source_code):
-        # Complex dealing with many cases, pylint: disable=too-many-branches,too-many-locals,too-many-statements
+        # Complex dealing with many cases, pylint: disable=too-many-branches
 
         description = anti_bloat_config.get("description", "description not given")
 
         # To allow detection if it did anything.
         change_count = 0
 
-        context = {"sys": sys, "os": os}
-
-        context_code = anti_bloat_config.get("context", "")
-        if type(context_code) in (tuple, list):
-            context_code = "\n".join(context_code)
-        context["version"] = self.getPackageVersion
-
-        # We trust the yaml files, pylint: disable=eval-used,exec-used
-        context_ready = not bool(context_code)
-
         for replace_src, replace_code in anti_bloat_config.get(
             "replacements", {}
         ).items():
@@ -316,36 +368,17 @@ which can and should be a top level package and then one choice, "error",
                 continue
 
             if replace_code:
-                if not context_ready:
-                    try:
-                        exec(context_code, context)
-                    except Exception as e:  # pylint: disable=broad-except
-                        self.sysexit(
-                            "Error, cannot execute context code '%s' due to: %s"
-                            % (context_code, e)
-                        )
-
-                    context_ready = True
-
-                if "__dirname__" in replace_code:
-                    context["__dirname__"] = self.locateModule(module_name)
-
-                try:
-                    replace_dst = eval(replace_code, context)
-                except Exception as e:  # pylint: disable=broad-except
-                    self.sysexit(
-                        "Error, cannot evaluate code '%s' in '%s' due to: %s"
-                        % (replace_code, context_code, e)
-                    )
+                replace_dst = self.evaluateExpression(
+                    full_name=module_name,
+                    expression=replace_code,
+                    config_name="module '%s' config 'replacements' " % module_name,
+                    extra_context=self._getContextCode(
+                        module_name=module_name, anti_bloat_config=anti_bloat_config
+                    ),
+                )
             else:
                 replace_dst = ""
 
-            if type(replace_dst) is not str:
-                self.sysexit(
-                    "Error, expression code '%s' needs to generate string, not %s"
-                    % (replace_code, type(replace_dst))
-                )
-
             old = source_code
             source_code = source_code.replace(replace_src, replace_dst)
 
@@ -375,17 +408,14 @@ which can and should be a top level package and then one choice, "error",
             append_code = "\n".join(append_code)
 
         if append_code:
-            if not context_ready:
-                exec(context_code, context)
-                context_ready = True
-
-            try:
-                append_result = eval(append_code, context)
-            except Exception as e:  # pylint: disable=broad-except
-                self.sysexit(
-                    "Error, cannot evaluate code '%s' in '%s' due to: %s"
-                    % (append_code, context_code, e)
-                )
+            append_result = self.evaluateExpression(
+                full_name=module_name,
+                expression=append_code,
+                config_name="module '%s' config 'append_code'" % module_name,
+                extra_context=self._getContextCode(
+                    module_name=module_name, anti_bloat_config=anti_bloat_config
+                ),
+            )
 
             source_code += "\n" + append_result
             change_count += 1
@@ -419,7 +449,7 @@ which can and should be a top level package and then one choice, "error",
 
         return source_code
 
-    def onModuleSourceCode(self, module_name, source_code):
+    def onModuleSourceCode(self, module_name, source_filename, source_code):
         for anti_bloat_config in self.config.get(module_name, section="anti-bloat"):
             if self.evaluateCondition(
                 full_name=module_name, condition=anti_bloat_config.get("when", "True")
@@ -435,14 +465,6 @@ which can and should be a top level package and then one choice, "error",
     def _onFunctionBodyParsing(
         self, module_name, anti_bloat_config, function_name, body
     ):
-        context = {}
-        context_code = anti_bloat_config.get("context", "")
-        if type(context_code) in (tuple, list):
-            context_code = "\n".join(context_code)
-
-        # We trust the yaml files, pylint: disable=eval-used,exec-used
-        context_ready = not bool(context_code)
-
         replace_code = anti_bloat_config.get("change_function", {}).get(function_name)
 
         if replace_code == "un-callable":
@@ -454,17 +476,15 @@ which can and should be a top level package and then one choice, "error",
         if replace_code is None:
             return False
 
-        if not context_ready:
-            exec(context_code, context)
-            context_ready = True
-
-        try:
-            replacement = eval(replace_code, context)
-        except Exception as e:  # pylint: disable=broad-except
-            self.sysexit(
-                "Error, cannot evaluate code '%s' in '%s' due to: %s"
-                % (replace_code, context_code, e)
-            )
+        replacement = self.evaluateExpression(
+            full_name=module_name,
+            expression=replace_code,
+            config_name="module '%s' config 'change_function' of '%s'"
+            % (module_name, function_name),
+            extra_context=self._getContextCode(
+                module_name=module_name, anti_bloat_config=anti_bloat_config
+            ),
+        )
 
         # Single node is required, extract the generated module body with
         # single expression only statement value or a function body.
@@ -505,10 +525,70 @@ which can and should be a top level package and then one choice, "error",
 
         return result
 
+    def _getModuleBloatModeOverrides(self, using_module_name, intended_module_name):
+        while 1:
+            config = self.config.get(using_module_name, section="anti-bloat")
+
+            if config:
+                for anti_bloat_config in config:
+                    bloat_mode_overrides = anti_bloat_config.get(
+                        "bloat-mode-overrides", ()
+                    )
+
+                    if not bloat_mode_overrides:
+                        continue
+
+                    if self.evaluateCondition(
+                        full_name=intended_module_name,
+                        condition=anti_bloat_config.get("when", "True"),
+                    ):
+                        for module_name, mode in bloat_mode_overrides.items():
+                            if module_name == intended_module_name:
+                                return mode
+
+            using_module_name = using_module_name.getPackageName()
+
+            if not using_module_name:
+                break
+
+        return None
+
     def onModuleRecursion(
-        self, module_name, module_filename, module_kind, using_module_name, source_ref
+        self,
+        module_name,
+        module_filename,
+        module_kind,
+        using_module_name,
+        source_ref,
+        reason,
     ):
-        # This will allow "unittest.mock" to pass these things.
+        # Quite a few special cases, but not really complex.
+        # pylint: disable=too-many-branches,too-many-locals
+
+        # First off, activate "no-follow" configurations.
+        for (
+            config_of_module_name,
+            no_follow,
+            description,
+        ) in self.getYamlConfigItemItems(
+            module_name=module_name,
+            section="anti-bloat",
+            item_name="no-follow",
+            decide_relevant=lambda key, value: True,
+            recursive=True,
+        ):
+            self.no_follows[no_follow] = (config_of_module_name, description)
+
+        # Do not even look at these. It's either included by a module that is in standard
+        # library, or included for a module in standard library.
+        if reason == "stdlib" or (
+            using_module_name is not None
+            and getModuleByName(using_module_name).reason == "stdlib"
+        ):
+            return
+
+        # This will allow "unittest.mock" to pass "unittest". It's kind of a hack and
+        # hopefully unusual.
         if module_name == "unittest.mock" and module_name not in self.handled_modules:
             return
 
@@ -524,6 +604,15 @@ which can and should be a top level package and then one choice, "error",
                 return
 
             if module_name.hasNamespace(handled_module_name):
+                if using_module_name is not None:
+                    override_mode = self._getModuleBloatModeOverrides(
+                        using_module_name=using_module_name,
+                        intended_module_name=intended_module_name,
+                    )
+
+                    if override_mode is not None:
+                        mode = override_mode
+
                 # Make sure the compilation aborts or warns if asked to
                 if mode == "error":
                     raise NuitkaForbiddenImportEncounter(
@@ -542,13 +631,20 @@ which can and should be a top level package and then one choice, "error",
                     )
 
                     if key not in self.warnings_given:
+                        if handled_module_name == intended_module_name:
+                            handled_module_name_desc = "'%s'" % handled_module_name
+                        else:
+                            handled_module_name_desc = (
+                                "'%s' (intending to avoid '%s')"
+                                % (handled_module_name, intended_module_name)
+                            )
+
                         self.warning(
                             """\
-Undesirable import of '%s' (intending to avoid '%s') in \
-'%s' (at '%s') encountered. It may slow down compilation."""
+Undesirable import of %s in '%s' (at '%s') encountered. It may \
+slow down compilation."""
                             % (
-                                handled_module_name,
-                                intended_module_name,
+                                handled_module_name_desc,
                                 using_module_name,
                                 source_ref.getAsString(),
                             ),
@@ -580,25 +676,43 @@ Undesirable import of '%s' (intending to avoid '%s') in \
                     )
 
         if using_module_name is not None:
-            config = self.config.get(using_module_name, section="anti-bloat")
 
-            if config:
-                for anti_bloat_config in config:
-                    match, reason = module_name.matchesToShellPatterns(
-                        anti_bloat_config.get("no-auto-follow", ())
-                    )
+            def decideRelevant(key, value):
+                # Only checking keys of configs, pylint: disable=unused-argument
 
-                    if match:
-                        if self.evaluateCondition(
-                            full_name=module_name,
-                            condition=anti_bloat_config.get("when", "True"),
-                        ):
+                return module_name.hasNamespace(key)
 
-                            return (
-                                False,
-                                "according to yaml 'no-auto-follow' configuration of '%s' and '%s'"
-                                % (using_module_name, reason),
-                            )
+            for (
+                config_of_module_name,
+                no_auto_follow,
+                description,
+            ) in self.getYamlConfigItemItems(
+                module_name=using_module_name,
+                section="anti-bloat",
+                item_name="no-auto-follow",
+                decide_relevant=decideRelevant,
+                recursive=True,
+            ):
+                assert module_name.hasNamespace(no_auto_follow), no_auto_follow
+
+                self.no_auto_follows[no_auto_follow] = description
+
+                return (
+                    False,
+                    "according to yaml 'no-auto-follow' configuration of '%s' for '%s'"
+                    % (config_of_module_name, no_auto_follow),
+                )
+
+        for no_follow_pattern, (
+            config_of_module_name,
+            description,
+        ) in self.no_follows.items():
+            if module_name.matchesToShellPattern(no_follow_pattern)[0]:
+                return (
+                    False,
+                    "according to yaml 'no-follow' configuration of '%s' for '%s'"
+                    % (config_of_module_name, no_follow_pattern),
+                )
 
         # Do not provide an opinion about it.
         return None
@@ -613,3 +727,22 @@ Undesirable import of '%s' (intending to avoid '%s') in \
 
             if module_name.hasNamespace(handled_module_name):
                 return "bytecode"
+
+    def onModuleCompleteSet(self, module_set):
+        # TODO: Maybe have an entry point that works on the set of names
+        # instead, we are not looking at the modules, and most plugins probably
+        # do not care.
+        module_names = set(module.getFullName() for module in module_set)
+
+        for module_name, description in self.no_auto_follows.items():
+            # Some are irrelevant, e.g. when registering to a module that would have to
+            # be used elsewhere.
+            if description == "ignore":
+                continue
+
+            if module_name not in module_names:
+                self.info(
+                    """\
+Not including '%s' automatically in order to avoid bloat, but this may cause: %s."""
+                    % (module_name, description)
+                )
