@@ -21,12 +21,15 @@ Here the small things that fit nowhere else and don't deserve their own module.
 
 """
 
+import ctypes
 import functools
 import os
 import sys
 import time
 from contextlib import contextmanager
 
+from nuitka.__past__ import WindowsError  # pylint: disable=I0021,redefined-builtin
+
 
 def getOS():
     if os.name == "nt":
@@ -98,6 +101,8 @@ def getLinuxDistribution():
 
         if os.path.exists("/etc/os-release"):
             result, base, version = _parseOsReleaseFileContents("/etc/os-release")
+        elif os.path.exists("/usr/lib/os-release"):
+            result, base, version = _parseOsReleaseFileContents("/usr/lib/os-release")
         elif os.path.exists("/etc/SuSE-release"):
             result, base, version = _parseOsReleaseFileContents("/etc/SuSE-release")
         elif os.path.exists("/etc/issue"):
@@ -154,6 +159,12 @@ def isFedoraBasedLinux():
     return (base or dist_name) == "Fedora"
 
 
+def isArchBasedLinux():
+    dist_name, base, _dist_version = getLinuxDistribution()
+
+    return (base or dist_name) == "Arch"
+
+
 def isAndroidBasedLinux():
     # spell-checker: ignore googlesource
     if not isLinux():
@@ -209,12 +220,15 @@ def isOpenBSD():
     return getOS() == "OpenBSD"
 
 
+def isBSD():
+    return "BSD" in getOS()
+
+
 _is_alpine = None
 
 
 def isAlpineLinux():
     if os.name == "posix":
-
         # Avoid repeated file system lookup, pylint: disable=global-statement
         global _is_alpine
         if _is_alpine is None:
@@ -331,6 +345,12 @@ def withNoSyntaxWarning():
         yield
 
 
+@contextmanager
+def withNoWarning():
+    with withWarningRemoved(Warning):
+        yield
+
+
 def decoratorRetries(
     logger, purpose, consequence, attempts=5, sleep_time=1, exception_type=OSError
 ):
@@ -390,3 +410,10 @@ Disable Anti-Virus, e.g. Windows Defender for build folders. Retrying after a se
         return retryingFunction
 
     return inner
+
+
+def raiseWindowsError(message):
+    raise WindowsError(
+        ctypes.GetLastError(),
+        "%s (%s)" % (message, ctypes.FormatError(ctypes.GetLastError())),
+    )
