@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -17,18 +17,19 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-# This test is using signals and will only work if PySide properly accepts
-# compiled functions as callables.
+""" This test is using signals and will only work if PySide properly accepts
+compiled functions as callables.
+"""
 
 # nuitka-project: --standalone
 # nuitka-project: --enable-plugin=pyside6
 
+# nuitka-skip-unless-imports: PySide6.QtCore
+
 from __future__ import print_function
 
 from PySide6.QtCore import QMetaObject, QObject, Signal, Slot
 
-# nuitka-skip-unless-imports: PySide6.QtCore
-
 
 class Communicate(QObject):
     speak = Signal(int)
